@@ -1,10 +1,13 @@
 use strict;
 package CPAN::Reporter;
-our $VERSION = '1.2009'; # VERSION
+
+our $VERSION = '1.2016';
 
 use Config;
 use Capture::Tiny qw/ capture tee_merged /;
-use CPAN 1.9301 ();
+use CPAN 1.94 ();
+#CPAN.pm was split into separate files in this version
+#set minimum to it for simplicity
 use CPAN::Version ();
 use File::Basename qw/basename dirname/;
 use File::Find ();
@@ -495,6 +498,22 @@ TRANSPORT_REQUIRED
         }
         else {
             $CPAN::Frontend->mywarn( "CPAN::Reporter: " . $tr->errstr . "\n");
+
+            if ( $config->{retry_submission} ) {
+                sleep(3);
+
+                $CPAN::Frontend->mywarn( "CPAN::Reporter: second attempt\n");
+                $tr->errstr('');
+
+                if ( $tr->send() ) {
+                    CPAN::Reporter::History::_record_history( $result )
+                        if not $is_duplicate;
+                }
+                else {
+                    $CPAN::Frontend->mywarn( "CPAN::Reporter: " . $tr->errstr . "\n");
+                }
+            }
+
         }
     }
     else {
@@ -504,6 +523,39 @@ TRANSPORT_REQUIRED
     return;
 }
 
+sub _report_timeout {
+    my $result = shift;
+    if ($result->{exit_value} == 9) {
+        my $config_obj = CPAN::Reporter::Config::_open_config_file();
+        my $config;
+        $config = CPAN::Reporter::Config::_get_config_options( $config_obj )
+            if $config_obj;
+
+        if ($config->{'_store_problems_in_dir'}) {
+            my $distribution = $result->{dist}->base_id;
+            my $file = "e9.$distribution.${\(time)}.$$.log";
+            if (open my $to_log_fh, '>>', $config->{'_store_problems_in_dir'}.'/'.$file) {
+                print $to_log_fh $distribution,"\n";
+                print $to_log_fh "stage: ",$result->{phase},"\n";
+                print $to_log_fh $Config{archname},"\n";
+                print $to_log_fh _report_text( $result );
+            } else {
+                $CPAN::Frontend->mywarn( "CPAN::Reporter: writing ".
+                    $config->{'_store_problems_in_dir'}.'/'.$file. " failed\n");
+            }
+        }
+        if ($config->{'_problem_log'}) {
+            my $distribution = $result->{dist}->base_id;
+            if (open my $to_log_fh, '>>', $config->{'_problem_log'}) {
+                print $to_log_fh "$result->{phase} $distribution $Config{archname}\n";
+            } else {
+                $CPAN::Frontend->mywarn( "CPAN::Reporter: writing ".
+                    $config->{'_store_problems_in_dir'}. " failed\n");
+            }
+        }
+    }
+}
+
 #--------------------------------------------------------------------------#
 # _downgrade_known_causes
 # Downgrade failure/unknown grade if we can determine a cause
@@ -526,6 +578,8 @@ sub _downgrade_known_causes {
     # get prereqs
     _expand_result( $result );
 
+    _report_timeout( $result );
+
     # if process was halted with a signal, just set for discard and return
     if ( $result->{exit_value} & 127 ) {
         $result->{grade} = 'discard';
@@ -544,7 +598,8 @@ sub _downgrade_known_causes {
         $harness_error++;
         last;
       }
-      if( $line =~ /Perl .*? required.*?--this is only/ims ||
+      if( $line =~ /(?<!skipped: )Perl .*? required.*?--this is only/ims ||
+           #?<! - quick hack for https://github.com/cpan-testers/CPAN-Reporter/issues/23
         $line =~ /Perl version .*? or higher required\. We run/ims || #EU::MM
         $line =~ /ERROR: perl: Version .*? is installed, but we need version/ims ||
         $line =~ /ERROR: perl \(.*?\) is installed, but we need version/ims ||
@@ -647,23 +702,23 @@ sub _expand_result {
 
 # Entries bracketed with "/" are taken to be a regex; otherwise literal
 my @env_vars= qw(
-    /PERL/
+    /HARNESS/
     /LC_/
+    /PERL/
+    /_TEST/
+    COMSPEC
+    INCLUDE
     LANG
     LANGUAGE
-    PATH
-    SHELL
-    COMSPEC
-    TERM
-    TEMP
-    TMPDIR
-    AUTOMATED_TESTING
-    /AUTHOR_TEST/
-    INCLUDE
-    LIB
     LD_LIBRARY_PATH
-    PROCESSOR_IDENTIFIER
+    LIB
     NUMBER_OF_PROCESSORS
+    PATH
+    PROCESSOR_IDENTIFIER
+    SHELL
+    TEMP
+    TERM
+    TMPDIR
 );
 
 sub _env_report {
@@ -897,7 +952,9 @@ sub _parse_test_harness {
 # _prereq_report
 #--------------------------------------------------------------------------#
 
-my @prereq_sections = qw/requires build_requires configure_requires/;
+my @prereq_sections = qw(
+  requires build_requires configure_requires opt_requires opt_build_requires
+);
 
 sub _prereq_report {
     my $dist = shift;
@@ -905,16 +962,20 @@ sub _prereq_report {
 
     # Extract requires/build_requires from CPAN dist
     my $prereq_pm = $dist->prereq_pm;
+
     if ( ref $prereq_pm eq 'HASH' ) {
-        # is it the new CPAN style with requires/build_requires?
-        if (join(q{ }, sort keys %$prereq_pm) eq "build_requires requires") {
-            $need{requires} = $prereq_pm->{requires}
-                if  ref $prereq_pm->{requires} eq 'HASH';
-            $need{build_requires} = $prereq_pm->{build_requires}
-                if ref $prereq_pm->{build_requires} eq 'HASH';
+        # CPAN 1.94 returns hash with requires/build_requires # so no need to support old style
+        foreach (values %$prereq_pm) {
+          if (defined && ref ne 'HASH') {
+             require Data::Dumper;
+             warn "Data error detecting prerequisites. Please report it to CPAN::Reporter bug tracker:";
+             warn Data::Dumper::Dumper($prereq_pm);
+             die "Stopping";
+          }
         }
-        else {
-            $need{requires} = $prereq_pm;
+
+        for my $sec ( @prereq_sections ) {
+            $need{$sec} = $prereq_pm->{$sec} if keys %{ $prereq_pm->{$sec} };
         }
     }
 
@@ -1075,6 +1136,43 @@ HERE
 
 );
 
+sub _comment_text {
+
+    # We assemble the completed comment as a series of "parts" which
+    # will get joined together
+    my @comment_parts;
+
+    # All automated testing gets a preamble
+    if ($ENV{AUTOMATED_TESTING}) {
+        push @comment_parts,
+            "this report is from an automated smoke testing program\n"
+            . "and was not reviewed by a human for accuracy"
+    }
+
+    # If a comment file is provided, read it and add it to the comment
+    my $confdir = CPAN::Reporter::Config::_get_config_dir();
+    my $comment_file = File::Spec->catfile($confdir, 'comment.txt');
+    if ( -d $confdir && -f $comment_file && -r $comment_file ) {
+        open my $fh, '<:encoding(UTF-8)', $comment_file or die($!);
+        my $text;
+        do {
+            local $/ = undef; # No record (line) seperator on input
+            defined( $text = <$fh> ) or die($!);
+        };
+        chomp($text);
+        push @comment_parts, $text;
+        close $fh;
+    }
+
+    # If we have an empty comment so far, add a default value
+    if (scalar(@comment_parts) == 0) {
+        push @comment_parts, 'none provided';
+    }
+
+    # Join the parts seperated by a blank line
+    return join "\n\n", @comment_parts;
+}
+
 sub _report_text {
     my $data = shift;
     my $test_log = join(q{},@{$data->{output}});
@@ -1083,10 +1181,8 @@ sub _report_text {
         my $max_k = int(MAX_OUTPUT_LENGTH/1000) . "K";
         $test_log .= "\n[Output truncated after $max_k]\n\n";
     }
-    # Flag automated report
-    my $default_comment = $ENV{AUTOMATED_TESTING}
-        ? "this report is from an automated smoke testing program\nand was not reviewed by a human for accuracy"
-        : "none provided" ;
+
+    my $comment_body = _comment_text();
 
     # generate report
     my $output = << "ENDREPORT";
@@ -1109,7 +1205,7 @@ TESTER COMMENTS
 
 Additional comments from tester:
 
-$default_comment
+$comment_body
 
 ------------------------------
 PROGRAM OUTPUT
@@ -1405,7 +1501,7 @@ TRANSPORT_ARGS
     if ( ! -r $args{id_file} ) {
         $CPAN::Frontend->mywarn( <<"TRANSPORT_ARGS" );
 
-CPAN::Reporter: Could not find Metabase tranport 'id_file' parameter
+CPAN::Reporter: Could not find Metabase transport 'id_file' parameter
 located at '$args{id_file}'.
 See documentation for proper configuration of the 'transport' setting.
 
@@ -1473,13 +1569,15 @@ sub _version_finder {
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 CPAN::Reporter - Adds CPAN Testers reporting to CPAN.pm
 
 =head1 VERSION
 
-version 1.2009
+version 1.2016
 
 =head1 SYNOPSIS
 
@@ -1574,6 +1672,28 @@ should copy it into the C<<< .cpanreporter >>> directory instead of creating
 a new one.  Profile files may be located outside the C<<< .cpanreporter >>>
 directory by following instructions in L<CPAN::Reporter::Config>.
 
+=head3 Default Test Comments
+
+This module puts default text into the "TESTER COMMENTS" section, typically,
+"none provided" if doing interactive testing, or, if doing smoke testing that
+sets CE<lt>$ENV{AUTOMATED_TESTING}E<gt> to a true value, "this report is from an
+automated smoke testing program and was not reviewed by a human for
+accuracy."  If CE<lt>CPAN::ReporterE<gt> is configured to allow editing of the
+report, this can be edited during submission.
+
+If you wish to override the default comment, you can create a file named
+CE<lt>comment.txtE<gt> in the configuration directory (typically C<<< .cpanreporter >>>
+under the user's home directory), with the default comment you would
+like to appear.
+
+Note that if your test is an automated smoke
+test (CE<lt>$ENV{AUTOMATED_TESTING}E<gt> is set to a true value), the smoke
+test notice ("this report is from an automated smoke testing program and
+was not reviewed by a human for accuracy") is included along with a blank
+line before your CE<lt>comment.txtE<gt>, so that it is always possible to
+distinguish automated tests from non-automated tests that use this
+module.
+
 =head2 Using CPAN::Reporter
 
 Once CPAN::Reporter is enabled and configured, test or install modules with
@@ -1641,6 +1761,9 @@ prior to sending it.
 
 =head1 BUGS
 
+Using command_timeout on Linux may cause problems. See
+L<https://rt.cpan.org/Ticket/Display.html?id=62310>
+
 Please report any bugs or feature using the CPAN Request Tracker.
 Bugs can be submitted through the web interface at
 L<http://rt.cpan.org/Dist/Display.html?Queue=CPAN-Reporter>
@@ -1689,7 +1812,7 @@ L<CPAN::Reporter::FAQ> -- hints and tips
 =head2 Bugs / Feature Requests
 
 Please report any bugs or feature requests through the issue tracker
-at L<https://rt.cpan.org/Public/Dist/Display.html?Name=CPAN-Reporter>.
+at L<https://github.com/cpan-testers/CPAN-Reporter/issues>.
 You will be notified automatically of any progress on your issue.
 
 =head2 Source Code
@@ -1697,14 +1820,66 @@ You will be notified automatically of any progress on your issue.
 This is open source software.  The code repository is available for
 public review and contribution under the terms of the license.
 
-L<https://github.com/dagolden/cpan-reporter>
+L<https://github.com/cpan-testers/CPAN-Reporter>
 
-  git clone git://github.com/dagolden/cpan-reporter.git
+  git clone https://github.com/cpan-testers/CPAN-Reporter.git
 
 =head1 AUTHOR
 
 David Golden <dagolden@cpan.org>
 
+=head1 CONTRIBUTORS
+
+=for stopwords Alexandr Ciornii Breno G. de Oliveira Christian Walde Ed J Joel Maslak Kent Fredric Matthew Musgrove Patrice Clement Reini Urban Scott Wiersdorf Slaven Rezic
+
+=over 4
+
+=item *
+
+Alexandr Ciornii <alexchorny@gmail.com>
+
+=item *
+
+Breno G. de Oliveira <garu@cpan.org>
+
+=item *
+
+Christian Walde <walde.christian@googlemail.com>
+
+=item *
+
+Ed J <mohawk2@users.noreply.github.com>
+
+=item *
+
+Joel Maslak <jmaslak@antelope.net>
+
+=item *
+
+Kent Fredric <kentfredric@gmail.com>
+
+=item *
+
+Matthew Musgrove <mr.muskrat@gmail.com>
+
+=item *
+
+Patrice Clement <monsieurp@gentoo.org>
+
+=item *
+
+Reini Urban <rurban@cpanel.net>
+
+=item *
+
+Scott Wiersdorf <scott@perlcode.org>
+
+=item *
+
+Slaven Rezic <slaven@rezic.de>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is Copyright (c) 2006 by David Golden.
