@@ -1,7 +1,7 @@
 use strict;
 package CPAN::Reporter::PrereqCheck;
 
-our $VERSION = '1.2014';
+our $VERSION = '1.2016';
 
 use ExtUtils::MakeMaker 6.36;
 use File::Spec;
@@ -52,16 +52,28 @@ sub _run {
 
             # get version from file or else report missing
             if ( defined $inst_file ) {
-                $have = MM->parse_version($inst_file);
-                $have = "0" if ! defined $have || $have eq 'undef';
+                $have = my $preliminary_version = MM->parse_version($inst_file);
+                $preliminary_version = "0" if ! defined $preliminary_version || $preliminary_version eq 'undef';
                 # report broken if it can't be loaded
                 # "select" to try to suppress spurious newlines
                 select DEVNULL; ## no critic
-                if ( ! _try_load( $mod, $have ) ) {
+                if ( ! _try_load( $mod, $preliminary_version ) ) {
                     select STDOUT; ## no critic
                     print "$mod 0 broken\n";
                     next;
                 }
+                # Now the module is loaded: if MM->parse_version previously failed to
+                # get the version, then we can now look at the value of the $VERSION
+                # variable.
+                if (! defined $have || $have eq 'undef') {
+                    no strict 'refs';
+                    my $mod_version = ${$mod.'::VERSION'};
+                    if (defined $mod_version) {
+                        $have = $mod_version;
+                    } else {
+                        $have = 0; # fallback
+                    }
+                }
                 select STDOUT; ## no critic
             }
             else {
@@ -134,7 +146,7 @@ sub _try_load {
     qw/mylib/,
 
     #do not return true value
-    qw/perlsecret/,
+    qw/perlsecret Alt::Crypt::RSA::BigInt/,
   );
 
   my %loading_conflicts = (
@@ -143,6 +155,12 @@ sub _try_load {
     'Dancer::Plugin::FlashNote' => ['Dancer::Plugin::FlashMessage'],
     'Dancer::Plugin::Mongoose' => ['Dancer::Plugin::DBIC'],
     'Dancer::Plugin::DBIC' => ['Dancer::Plugin::Mongoose'],
+    'Test::BDD::Cucumber::Loader' => ['Test::Exception', 'Test::MockObject'], #works in different order
+    'Test::Mock::LWP::UserAgent' => ['HTTP::Response'],
+    'Test::SharedFork' => ['threads'], #dies if $INC{'threads.pm'}
+    'Test::TCP' => ['threads'], #loads Test::SharedFork
+    'Test::Fake::HTTPD' => ['threads'], #loads Test::SharedFork
+    #Note: Test::Perl::Critic and other modules load threads, so reordering will not help
   ); #modules that conflict with each other
   
   my %load_before = (
@@ -217,7 +235,7 @@ CPAN::Reporter::PrereqCheck - Modulino for prerequisite tests
 
 =head1 VERSION
 
-version 1.2014
+version 1.2016
 
 =head1 SYNOPSIS
 
